/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tasks.v  
   Created on           : 5th March 99
   Function             : contains tasks for the PCI test bench
   Author               : Ramaprasad K V,Pramod G
   Revision             : 
   Verified             : yes/no
   Modification History : 
   19 Mar 99              Modifications for giving parity also 
   19 May 99              Made tasks for i/o rd & wr , 64 bit mem rd & wr 
   25 May 99              Made tasks for mem_wr_step  -- Pramod
   26 May 99              Made tasks for mem_wr_wait  -- Pramod
   29 May 99              Made tasks for tgt_disconnect -- Pramod
   Instantiated in      : 

   Instantiates         :
=========================================================================*/

parameter tsu = 23  ; // (30 - 7 (setup) = 23)


/****************************/
// write a value to a configuration reg 
task config_wr ;
   input [31:0] reg_addr ;
   input [31:0] data_to_write ;
   
   reg PAR1;
   
   begin 
   @(posedge I_CLK)
   #tsu I_IDSEL = 1;
      AD[63:32] = 32'bz ;
      AD[31:0]  = reg_addr ;
      I_CBE_N[3:0] = 4'hB ; //  command for config write 
      I_FRAME_N = 0 ;
      PAR1 = gen_prty(reg_addr,I_CBE_N[3:0] ) ;
   @(posedge I_CLK)  
   #tsu I_IDSEL = 0 ;
      I_CBE_N[7:0] = 0 ;
	  AD[63:32] = 32'bz ;
      AD[31:0]  = data_to_write ;
      I_IRDY_N = 0 ;
      I_FRAME_N = 1 ;
      PAR = PAR1;
	  $display ("At time %d writing %h at register address %h", $time,data_to_write,reg_addr) ;
   @(posedge I_CLK)
   #tsu 
      PAR = gen_prty(data_to_write,I_CBE_N[3:0]) ;
   @(posedge TRDY_N)
      #tsu   I_IRDY_N = 1 ;
           AD = 64'bz ;
   end
endtask 

/****************************/
//read from a configuration register -- modify with expected data --
task config_rd  ;
   input [31:0] reg_addr ;
   output [31:0] data_from_reg ;  

   reg [31:0] data_from_reg ;

   begin 
   @(posedge I_CLK)
   #tsu I_IDSEL = 1;
	  AD[63:32] = 32'bz ;
      AD[31:0]  = reg_addr ;
      I_CBE_N[3:0] = 4'hA ; //  command for config read 
      I_FRAME_N = 0 ;

   @(posedge I_CLK)
   #tsu I_IDSEL = 0 ;
      I_CBE_N[3:0] = 0 ;
      AD = 64'bz ;
      I_IRDY_N = 0 ;
      I_FRAME_N = 1 ;
      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if (TRDY_N == 1'b0) // transfer should have taken place at this edge 
         data_from_reg = IO_AD[31:0] ;
      #tsu I_IRDY_N = 1;
	  $display ("At time %d read ; register address %h data %h ", $time,reg_addr,data_from_reg) ;
   end 
endtask 

/****************************/
// The following task is only for configuration cycles debug ; 
// Keeps frame asserted for more than 1 cycle 

task long_config_wr ;
   input [31:0] reg_addr ;
   input [31:0] data_to_write ;

   begin 
   @(posedge I_CLK)
   #tsu I_IDSEL = 1;
	  AD[63:32] = 32'bz ;
      AD[31:0]  = reg_addr ;
      I_CBE_N[3:0] = 4'hB ; //  command for config write 
      I_FRAME_N = 0 ;
   @(posedge I_CLK)  
   #tsu I_IDSEL = 0 ;
      I_CBE_N[3:0] = 0 ;
	  AD[63:32] = 32'bz ;
      AD[31:0]  = data_to_write ;
      I_IRDY_N = 0 ;
      I_FRAME_N = 0 ;
	  $display ("At time %d writing %h at register address %h", $time,data_to_write,reg_addr) ;
   @(posedge TRDY_N)
	  #100 I_FRAME_N = 1 ; //check 
      #tsu   I_IRDY_N = 1 ;
           AD = 64'bz ;
   end
endtask 

/****************************/
// The following task sets the memory base address for a target  

task set_mem_base_addr ;
   input [31:0] base_addr ;
   reg [31:0] read_data ;

   begin 

   // memory base low register address = 10h
   config_wr(32'h10 , base_addr);
   config_rd(32'h10,read_data) ;

   if (read_data[31:12] == base_addr[31:12])  // lower 12 bits are read only
      $display("### The memory base address set to %h ", base_addr );
   else
      $display ("#### ERROR in writing the memory base address");
   end
endtask 

/****************************/
// sets the io base address for the target 
task set_io_base_addr ;
   input [31:0] base_addr ;
   reg [31:0] read_data ;

   begin 

   // io base low register address = 1Ch
   config_wr(32'h18 , base_addr);////address changed to 18h from 1ch 
   config_rd(32'h18,read_data) ;////address changed to 18h from 1ch

   if (read_data[31:8] == base_addr[31:8])  // lower 8 bits are read only
      $display("### The io base address set to %h ", base_addr );
   else
      $display ("#### ERROR in writing the io base address");
   end
endtask


/****************************/
// write a single word to the target  -- 32 bit data width

task mem_wr ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ; 

   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 1; // indicating single word write 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
         #tsu 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[3:0] = 4'bz ;
         end
   end 
endtask 

/****************************/
// invalid address
// write a single word to the target
// -- 32 bit data width

task mem_wr_invalid_addr ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ; 
   integer i;  // for using in for loop 
   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 1; // indicating single word write 
      I_IRDY_N = 0 ;

//    wait(!TRDY_N) ; // for trdy to be low
   for(i=0;i<7;i=i+1)
      @(posedge I_CLK)
      if(DEVSEL_N == 1'b1) 
         $display (" DEVSEL_N = %d %d ",DEVSEL_N,i+2);
      else   
         $display ("%d error in address rejection",i+2); 
   
   end 
endtask 

/****************************/
// write a burst data to the target  -- 32 bit data width
// increment data for each write 

task mem_wr_burst ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ;
   input [3:0] burst_len ;  // for check Cn be a different #

   reg [3:0] length_count ;
   reg [31:0] data ;
   begin 

   // address- command phase 

   @(posedge I_CLK)
   #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      length_count = burst_len ;
      data = wr_data ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_IRDY_N = 0 ;
      length_count = length_count -1 ;

      while (length_count > 1)
      begin 
      
         @(posedge I_CLK)
            if (!TRDY_N )  // one write has happened, give next data 
            begin 
            #tsu  data = data +1 ;
                  length_count = length_count -1 ;
                  AD[31:0] = data ;
            end 
      end  
   wait(!TRDY_N)
   @(posedge I_CLK)  
      #tsu
         I_FRAME_N =  1 ; // indicating last transfer
         data =data + 1;
         AD[31:0] = data;
       

   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         #tsu 
            begin 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[3:0] = 4'bz ;
         end

   end 
endtask


/****************************/
//read a single 32 bit word from memory 
task mem_rd ;

   input [31:0] rd_addr ;

   reg [31:0] data_recd ;
   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = rd_addr ;
      I_CBE_N[3:0] = 4'h6 ; // "0110" = mem_read command  
      I_FRAME_N = 0 ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 1; // indicating single word read 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
            data_recd = IO_AD ;
         #tsu 
            I_IRDY_N = 1 ;
            I_CBE_N[3:0] = 4'bz ;
            $display ("Read address = %h received data = %h", rd_addr, data_recd);
         end
   end 
endtask 


/****************************/
// Read a burst data from memory 
task mem_rd_burst ;

   input [31:0] rd_addr ;
   input [3:0] burst_len ;

   reg [31:0] data_recd ;
   reg [3:0] length_count ;

   begin 

   // address/command phase 

   @(posedge I_CLK)
      #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = rd_addr ;
      I_CBE_N[3:0] = 4'h6 ; // "0110" = mem_read command  
      I_FRAME_N = 0 ;
      length_count = burst_len ;

   // data phase 

   @(posedge I_CLK)
      #tsu
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[3:0] = 4'b0 ;
      I_IRDY_N = 0 ;

   wait(!TRDY_N) // for TRDY to be asserted 

   while (length_count > 1)
   begin
   @(posedge I_CLK)
      if (!TRDY_N)
      begin
         data_recd = IO_AD ;
         $display ("&&1 Read address = %h received data = %h", rd_addr, data_recd);
         length_count = length_count - 1;
         rd_addr = rd_addr + 4 ;
         if (length_count == 1) 
         begin
            #tsu I_FRAME_N   = 1'b1 ;
            $display("&&2 frame deasserted");
         end 
      end 
   end 
 
   wait (!TRDY_N)
   @(posedge I_CLK)
      if (!TRDY_N)
      begin
         data_recd = IO_AD ;
         $display ("&&3 last Read address = %h received data = %h",rd_addr, data_recd);
         # tsu I_IRDY_N <= 1'b1 ;
         $display ("End of burst reads of length = %d ", burst_len );
      end 
   end 
endtask

/****************************/
// a 64 bit wide write, address is still 32 bit
task mem_wr_64_dat ;
   input [31:0] wr_addr ;
   input [63:0] wr_data ; 

   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;
   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:0] = wr_data ;
      I_CBE_N[7:0] = 8'b0 ;
      I_FRAME_N = 1; // indicating single word write 
      I_REQ64_N = 1; // follows frame 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
         #tsu 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[7:0] = 8'bz ;
         end
   end 
endtask 

/****************************/
// a burst of 64 bit wide writes , address is still 32 bit

task mem_wr_64_dat_burst ;
   input [31:0] wr_addr ;
   input [63:0] wr_data ;
   input [3:0] burst_len ;  

   reg [3:0] length_count ;
   reg [63:0] data ;
   begin 

   // address- command phase 

   @(posedge I_CLK)
   #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;
      length_count = burst_len ;
      data = wr_data ;

   // data phase  -- NOW DOES NOT LOOK TO ACK64 

   @(posedge I_CLK)
   #tsu 
      AD[63:0] = wr_data;
      I_CBE_N[7:0] = 8'b0 ;
      I_IRDY_N = 0 ;
      length_count = length_count -1 ;

      while (length_count > 1)
      begin 
      
         @(posedge I_CLK)
            if (!TRDY_N )  // one write has happened, give next data 
            begin 
            #tsu  data = data +1 ;
                  length_count = length_count -1 ;
                  AD[63:0] = data ;
            end 
      end  
   wait(!TRDY_N)
   @(posedge I_CLK)  
      #tsu
         I_FRAME_N =  1 ; // indicating last transfer
         I_REQ64_N = 1;
         data =data + 1;
         AD[63:0] = data;
       

   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         #tsu 
            begin 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[7:0] = 8'bz ;
         end

   end 
endtask

/****************************/
// a 64 bit wide write ,DAC

task mem_wr_64 ;
   input [63:0] wr_addr ;
   input [63:0] wr_data ; 

   begin 

   // first address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = wr_addr[63:32] ;
      AD[31:0]  = wr_addr[31:0] ;
      I_CBE_N[3:0] = 4'hD ; // "1101" = DAC ;
      I_CBE_N[7:4] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // second address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = wr_addr[63:32];  
      AD[31:0]  = wr_addr[63:32];
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_CBE_N[7:4] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // data phase 
   @(posedge I_CLK)
   #tsu 
      AD[63:0] = wr_data ;
      I_CBE_N[7:0] = 8'b0 ;  // modified from 4'b to 8'b
      I_FRAME_N = 1; // indicating single word write 
      I_REQ64_N = 1; // follows frame 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
         #tsu 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[7:0] = 8'bz ;
         end
   end 
endtask 

/****************************/
// a burst of 64 bit wide writes , DAC

task mem_wr_64_burst ;
   input [63:0] wr_addr ;
   input [63:0] wr_data ;
   input [3:0] burst_len ;  

   reg [3:0] length_count ;
   reg [63:0] data ;
   begin 

   // first address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = wr_addr[63:32] ;
      AD[31:0]  = wr_addr[31:0] ;
      I_CBE_N[3:0] = 4'hD ; // "1101" = DAC ;
      I_CBE_N[7:4] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;
      length_count = burst_len ;
      data = wr_data ;

   // second address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = wr_addr[63:32]; 
      AD[31:0]  = wr_addr[63:32];
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_CBE_N[7:4] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // data phase  -- NOW DOES NOT LOOK TO ACK64 

   @(posedge I_CLK)
   #tsu 
      AD[63:0] = wr_data;
      I_CBE_N[7:0] = 8'b0 ;
      I_IRDY_N = 0 ;
      length_count = length_count -1 ;

      while (length_count > 1)
      begin 
      
         @(posedge I_CLK)
            if (!TRDY_N )  // one write has happened, give next data 
            begin 
            #tsu  data = data +1 ;
                  length_count = length_count -1 ;
                  AD[63:0] = data ;
            end 
      end  
   wait(!TRDY_N)
   @(posedge I_CLK)  
      #tsu
         I_FRAME_N =  1 ; // indicating last transfer
         I_REQ64_N = 1;
         data =data + 1;
         AD[63:0] = data;
       

   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         #tsu 
            begin 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[7:0] = 8'bz ;
         end

   end 
endtask

/****************************/
// a 32 bit wide write , DAC

task mem_wr_64_addr ;
   input [63:0] wr_addr ;
   input [31:0] wr_data ; 

   begin 

   // first address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = wr_addr[63:32] ;
      AD[31:0]  = wr_addr[31:0] ;
      I_CBE_N[3:0] = 4'hD ; // "1101" = DAC ;
      I_CBE_N[7:4] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // second address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = wr_addr[63:32];  
      AD[31:0]  = wr_addr[63:32];
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_CBE_N[7:4] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // data phase 
   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 1; // indicating single word write 
      I_REQ64_N = 1; // follows frame 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
         #tsu 
            I_IRDY_N = 1 ;
            AD[31:0] = 64'bz ;
            I_CBE_N[3:0] = 4'bz ;
         end
   end 
endtask 

/****************************/
// a burst of 32 bit wide writes , DAC

task mem_wr_64_addr_burst ;
   input [63:0] wr_addr ;
   input [31:0] wr_data ;
   input [3:0] burst_len ;  

   reg [3:0] length_count ;
   reg [31:0] data ;
   begin 

   // first address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = wr_addr[63:32] ;
      AD[31:0]  = wr_addr[31:0] ;
      I_CBE_N[3:0] = 4'hD ; // "1101" = DAC ;
      I_CBE_N[7:4] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

      length_count = burst_len ;
      data = wr_data ;

   // second address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = wr_addr[63:32];  
      AD[31:0]  = wr_addr[63:32];  
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_CBE_N[7:4] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // data phase  -- NOW DOES NOT LOOK TO ACK64 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_IRDY_N = 0 ;
      length_count = length_count -1 ;

      while (length_count > 1)
      begin 
      
         @(posedge I_CLK)
            if (!TRDY_N )  // one write has happened, give next data 
            begin 
            #tsu  data = data +1 ;
                  length_count = length_count -1 ;
                  AD[31:0] = data ;
            end 
      end  
   wait(!TRDY_N)
   @(posedge I_CLK)  
      #tsu
         I_FRAME_N =  1 ; // indicating last transfer
         I_REQ64_N = 1;
         data =data + 1;
         AD[31:0] = data;
       

   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         #tsu 
            begin 
            I_IRDY_N = 1 ;
            AD[31:0] = 64'bz ;
            I_CBE_N[3:0] = 4'bz ;
         end

   end 
endtask

/****************************/
// Read a data from memory  with 64 bits data width 
task mem_rd_64_dat ;

   input [31:0] rd_addr ;

   reg [63:0] data_recd ;
   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = rd_addr ;
      I_CBE_N[3:0] = 4'h6 ; // "0110" = mem_read command  
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ; // indicating request for 64 bit data path

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 1; // indicating single word read 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
            data_recd = IO_AD ;
         #tsu 
            I_IRDY_N = 1 ;
            I_CBE_N[3:0] = 4'bz ;
            $display ("Read address = %h received data = %h", rd_addr, data_recd);
         end
   end 
endtask 

/****************************/
// Read a burst data from memory  with 64 bits data width 
task mem_rd_64_dat_burst ;

   input [31:0] rd_addr ;
   input [3:0] burst_len ;

   reg [63:0] data_recd ;
   reg [3:0] length_count ;

   begin 

   // address/command phase 

   @(posedge I_CLK)
      #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = rd_addr ;
      I_CBE_N[3:0] = 4'h6 ; // "0110" = mem_read command  
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;
      length_count = burst_len ;

   // data phase 

   @(posedge I_CLK)
      #tsu
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[7:0] = 8'b0 ;
      I_IRDY_N = 0 ;

   wait(!TRDY_N) // for TRDY to be asserted 

   while (length_count > 1)
   begin
   @(posedge I_CLK)
      if (!TRDY_N)
      begin
         data_recd = IO_AD ;
         $display ("&&1 Read address = %h received data = %h", rd_addr, data_recd);
         length_count = length_count - 1;
         rd_addr = rd_addr + 8 ; //For 64 bit data
         if (length_count == 1) 
         begin
            I_FRAME_N   = 1'b1 ;
            I_REQ64_N   = 1'b1 ;
            $display("&&2 frame & req64 deasserted");
         end 
      end 
   end 
   wait (!TRDY_N)
   @(posedge I_CLK)
      if (!TRDY_N)
       begin
         $display ("&&3 last Read address = %h received data = %h",rd_addr, data_recd);
         # tsu I_IRDY_N <= 1'b1 ;
         $display ("End of burst reads of length = %d ", burst_len );
      end 
   end 
endtask

/****************************/
// Read a data from memory  with 64 bits data width -- DAC 
task mem_rd_64 ;

   input [63:0] rd_addr ;

   reg [63:0] data_recd ;
   begin 

   // first address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = rd_addr[63:32] ;
      AD[31:0]  = rd_addr[31:0] ;
      I_CBE_N[3:0] = 4'hD ; // "1101" = DAC ;
      I_CBE_N[7:4] = 4'h6 ; // "0110" = mem_rd ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // second address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = rd_addr[63:32]; 
      AD[31:0]  = rd_addr[63:32]; 
      I_CBE_N[3:0] = 4'h6 ; // "0110" = mem_rd ;
      I_CBE_N[7:4] = 4'h6 ; // "0110" = mem_rd ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;


   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[7:0] = 8'b0 ;
      I_FRAME_N = 1; // indicating single word read 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
            data_recd = IO_AD ;
         #tsu 
            I_IRDY_N = 1 ;
            I_CBE_N[7:0] = 8'bz ;
            $display ("Read address = %h received data = %h", rd_addr, data_recd);
         end
   end 
endtask 

/****************************/
// Read a burst data from memory  with 64 bits data width -- DAC 
task mem_rd_64_burst ;

   input [63:0] rd_addr ;
   input [3:0] burst_len ;

   reg [63:0] data_recd ;
   reg [3:0] length_count ;

   begin 

   // first address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = rd_addr[63:32] ;
      AD[31:0]  = rd_addr[31:0] ;
      I_CBE_N[3:0] = 4'hD ; // "1101" = DAC ;
      I_CBE_N[7:4] = 4'h6 ; // "0110" = mem_rd ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;
      length_count =  burst_len;  
   // second address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = rd_addr[63:32]; 
      AD[31:0]  = rd_addr[63:32];  
      I_CBE_N[3:0] = 4'h6 ; // "0110" = mem_rd ;
      I_CBE_N[7:4] = 4'h6 ; // "0110" = mem_rd ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // data phase 

   @(posedge I_CLK)
      #tsu
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[7:0] = 8'b0 ;
      I_IRDY_N = 0 ;

   wait(!TRDY_N) // for TRDY to be asserted 

   while (length_count > 1)
   begin
   @(posedge I_CLK)
      if (!TRDY_N)
      begin
         data_recd = IO_AD ;
         $display ("&&1 Read address = %h received data = %h", rd_addr, data_recd);
         length_count = length_count - 1;
         rd_addr = rd_addr + 8 ; //For 64 bit data
         if (length_count == 1) 
         begin
            I_FRAME_N   = 1'b1 ;
            I_REQ64_N   = 1'b1 ;
            $display("&&2 frame & req64 deasserted");
         end 
      end 
   end 
 
   wait (!TRDY_N)
   @(posedge I_CLK)
      if (!TRDY_N)
      begin
         data_recd = IO_AD ;
         $display ("&&3 last Read address = %h received data = %h",rd_addr, data_recd);
         # tsu I_IRDY_N <= 1'b1 ;
         $display ("End of burst reads of length = %d ", burst_len );
      end 
   end 
endtask

/****************************/
// Read a data from memory  with 32 bits data width -- DAC 
task mem_rd_64_addr ;

   input [63:0] rd_addr ;

   reg [31:0] data_read ;
   begin 

   // first address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = rd_addr[63:32] ;
      AD[31:0]  = rd_addr[31:0] ;
      I_CBE_N[3:0] = 4'hD ; // "1101" = DAC ;
      I_CBE_N[7:4] = 4'h6 ; // "0110" = mem_rd ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // second address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = rd_addr[63:32];  
      AD[31:0]  = rd_addr[63:32];  
      I_CBE_N[3:0] = 4'h6 ; // "0110" = mem_rd ;
      I_CBE_N[7:4] = 4'h6 ; // "0110" = mem_rd ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 1; // indicating single word read 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
            data_read = IO_AD ;
         #tsu 
            I_IRDY_N = 1 ;
            I_CBE_N[3:0] = 4'bz ;
            $display ("Read address = %h received data = %h", rd_addr, data_read);
         end
   end 
endtask 

/****************************/
// Read a burst data from memory  with 32 bits data width -- DAC 
task mem_rd_64_addr_burst ;

   input [63:0] rd_addr ;
   input [3:0] burst_len ;

   reg [31:0] data_recd ;
   reg [3:0] length_count ;

   begin 

   // first address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = rd_addr[63:32] ;
      AD[31:0]  = rd_addr[31:0] ;
      I_CBE_N[3:0] = 4'hD ; // "1101" = DAC ;
      I_CBE_N[7:4] = 4'h6 ; // "0110" = mem_rd ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;
      length_count = burst_len;
   // second address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = rd_addr[63:32];  
      AD[31:0]  = rd_addr[63:32];  
      I_CBE_N[3:0] = 4'h6 ; // "0110" = mem_rd ;
      I_CBE_N[7:4] = 4'h6 ; // "0110" = mem_rd ;
      I_FRAME_N = 0 ;
      I_REQ64_N = 0 ;

   // data phase 

   @(posedge I_CLK)
      #tsu
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[3:0] = 4'b0 ;
      I_IRDY_N = 0 ;

   wait(!TRDY_N) // for TRDY to be asserted 

   while (length_count > 1)
   begin
   @(posedge I_CLK)
      if (!TRDY_N)
      begin
         data_recd = IO_AD ;
         $display ("&&1 Read address = %h received data = %h", rd_addr, data_recd);
         length_count = length_count - 1;
         rd_addr = rd_addr + 4 ;
         if (length_count == 1) 
         begin
            I_FRAME_N   = 1'b1 ;
            I_REQ64_N   = 1'b1 ;
            $display("&&2 frame & req64 deasserted");
         end 
      end 
   end 
 
   wait (!TRDY_N)
   @(posedge I_CLK)
      if (!TRDY_N)
      begin
         data_recd = IO_AD ;
         $display ("&&3 last Read address = %h received data = %h",rd_addr, data_recd);
         # tsu I_IRDY_N <= 1'b1 ;
         $display ("End of burst reads of length = %d ", burst_len );
      end 
   end 
endtask

/****************************/
function gen_prty ;
input [31:0] indata ;
input [3:0] CBE ;

reg [35:0] in_data; 
begin
   in_data = {indata[31:0],CBE[3:0]};
   gen_prty = ^in_data ;

end 
endfunction


/* Actually io_wr and io_burst should be very similar to mem_wr and mem_wr_burst
   task io_wr 
   task io_wr_burst ;
   task io_rd ;
   task io_rd_burst ;
*/

/****************************/
// write a single word to the I/O space of target  -- 32 bit data width
task io_wr ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ; 

   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h3 ; // "0011" = io_wr ;
      I_FRAME_N = 0 ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 1; // indicating single word write 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
         #tsu 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[3:0] = 4'bz ;
         end
   end 
endtask 

/****************************/
// write a burst of words to the I/O space of target 
task io_wr_burst ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ;
   input [3:0] burst_len ;  // for check Cn be a different #

   reg [3:0] length_count ;
   reg [31:0] data ;
   begin 

   // address- command phase 

   @(posedge I_CLK)
   #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h3 ; // "0011" = io_wr ;
      I_FRAME_N = 0 ;
      length_count = burst_len ;
      data = wr_data ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_IRDY_N = 0 ;
      length_count = length_count -1 ;

      while (length_count > 1)
      begin 
      
         @(posedge I_CLK)
            if (!TRDY_N )  // one write has happened, give next data 
            begin 
            #tsu  data = data +1 ;
                  length_count = length_count -1 ;
                  AD[31:0] = data ;
            end 
      end  
   wait(!TRDY_N)
   @(posedge I_CLK)  
      #tsu
         I_FRAME_N =  1 ; // indicating last transfer
         data =data + 1;
         AD[31:0] = data;
       

   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         #tsu 
            begin 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[3:0] = 4'bz ;
         end

   end 
endtask

/****************************/
//read a single 32 bit word from I/O space 
task io_rd ;

   input [31:0] rd_addr ;

   reg [31:0] data_recd ;
   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = rd_addr ;
      I_CBE_N[3:0] = 4'h2 ; // "0010" = io_read command ; // 
      I_FRAME_N = 0 ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 1; // indicating single word read 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
            data_recd = IO_AD ;
         #tsu 
            I_IRDY_N = 1 ;
            I_CBE_N[3:0] = 4'bz ;
            $display ("Read address = %h received data = %h", rd_addr, data_recd);
         end
   end 
endtask 

/****************************/
// Read a burst data from I/O space 
task io_rd_burst ;

   input [31:0] rd_addr ;
   input [3:0] burst_len ;

   reg [31:0] data_recd ;
   reg [3:0] length_count ;

   begin 

   // address/command phase 

   @(posedge I_CLK)
      #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = rd_addr ;
      I_CBE_N[3:0] = 4'h2 ; // "0010" = io_read command 
      I_FRAME_N = 0 ;
      length_count = burst_len ;

   // data phase 

   @(posedge I_CLK)
      #tsu
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[3:0] = 4'b0 ;
      I_IRDY_N = 0 ;

   wait(!TRDY_N) // for TRDY to be asserted 

   while (length_count > 1)
   begin
   @(posedge I_CLK)
      if (!TRDY_N)
      begin
         data_recd = IO_AD ;
         $display ("&&1 Read address = %h received data = %h", rd_addr, data_recd);
         length_count = length_count - 1;
         rd_addr = rd_addr + 4 ;
         if (length_count == 1) 
         begin
            #tsu I_FRAME_N   = 1'b1 ;
            $display("&&2 frame deasserted");
         end 
      end 
   end 
 
   wait (!TRDY_N)
   @(posedge I_CLK)
      if (!TRDY_N)
      begin
         data_recd = IO_AD ;
         $display ("&&3 last Read address = %h received data = %h",rd_addr, data_recd);
         # tsu I_IRDY_N <= 1'b1 ;
         $display ("End of burst reads of length = %d ", burst_len );
      end 
   end 
endtask

/****************************/
// addr & data stepping
// a 32 bit wide write , address is 32 bit

task mem_wr_step ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ; 

   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 1 ;  // address stepping

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 1 ;  // address stepping

  @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;

   // data phase 
   @(posedge I_CLK)
   #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0] = wr_data  ;
      I_CBE_N[3:0] = 4'b0 ;  
      I_FRAME_N = 0; // indicating single word write 
      I_IRDY_N = 1 ;   // data stepping                  
   @(posedge I_CLK)
   #tsu 
      AD[31:0] = wr_data  ;
      I_CBE_N[3:0] = 4'b0 ;  
      I_FRAME_N = 1; // indicating single word write 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
         #tsu 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[7:0] = 8'bz ;
         end
   end 
endtask 


/****************************/
// write a burst data to the target  -- 32 bit data width
// increment data for each write 
// assert I_abort_from_app signal to check whether aborting correctly
task mem_wr_burst_abort ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ;
   input [3:0] burst_len ;  // for check Cn be a different #

   reg [3:0] length_count ;
   reg [31:0] data ;
   begin 

   // address- command phase 

   @(posedge I_CLK)
   #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      length_count = burst_len ;
      data = wr_data ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_IRDY_N = 0 ;
      length_count = length_count -1 ;

      while ((length_count > 1) & (~I_abort_from_app))
      begin 
      
         @(posedge I_CLK)
            if (!TRDY_N )  // one write has happened, give next data 
            begin 
            #tsu  data = data +1 ;
                  if(length_count == 4'd3 || length_count == 4'd2)
                     I_abort_from_app = 1'b1;
                  length_count = length_count -1 ;
                  AD[31:0] = data ;
            end 
      end  
     
   @(posedge I_CLK) //newly added
      if(TRDY_N & ~STOP_N & DEVSEL_N)
         begin
         $display("t= %0d O_TRDY_N=%b O_STOP_N = %b O_DEVSEL =%b",
                $time,TRDY_N, STOP_N , DEVSEL_N);
         $display("abort properly");         
         end
      else
         begin
         $display("t= %0d O_TRDY_N=%b O_STOP_N = %b O_DEVSEL =%b",
                $time,TRDY_N, STOP_N , DEVSEL_N);
         $display("error in abort");         
         end  
   @(posedge I_CLK)  
      #tsu
         I_FRAME_N =  1 ; // indicating last transfer
         I_IRDY_N  = 0 ;
      if(TRDY_N & ~STOP_N & DEVSEL_N)
         begin
         $display("t= %0d O_TRDY_N=%b O_STOP_N = %b O_DEVSEL =%b",
                $time,TRDY_N, STOP_N , DEVSEL_N);
         $display("abort properly");         
         end
      else
         begin
         $display("t= %0d O_TRDY_N=%b O_STOP_N = %b O_DEVSEL =%b",
                $time,TRDY_N, STOP_N , DEVSEL_N);
         $display("error in abort");         
         end  
         
       
   @(posedge I_CLK) 
            begin 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[3:0] = 4'bz ;
            $display("t= %0d O_TRDY_N=%b O_STOP_N = %b O_DEVSEL =&b",
                     $time,TRDY_N, STOP_N , DEVSEL_N);
            
         end
   I_abort_from_app = 1'b0;
   end 
endtask


/****************************/
// write a burst of single words to the target  -- 32 bit data width
// wait states included
task mem_wr_wait ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ; 
   integer i;
   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'hC ;
      I_FRAME_N = 0;  
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low  
   @(posedge I_CLK)   // 1st data transfer
         begin 
         #tsu 
            I_IRDY_N = 1 ;      // wait state
            AD[63:32] = 32'bz ;
            I_CBE_N[3:0] = 4'b0 ;
         end
   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data + 1;
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 0;  
      I_IRDY_N = 0 ;
      wait(!TRDY_N) ; // for trdy to be low

   @(posedge I_CLK)   // 2nd data transfer
         begin 
         #tsu 
            I_IRDY_N = 1 ;          // wait state
            AD[63:32] = 32'bz ;
            AD[31:0] = wr_data + 1; // keep previous data
            I_CBE_N[3:0] = 4'b0 ;
         end
   @(posedge I_CLK)
   begin
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data + 2;
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 0;  
      I_IRDY_N = 0 ;
      wait(!TRDY_N) ; // for trdy to be low
   end 
   // wait more than 5 cycles -- not allowed by design -- max_ini_lat = 5 , now
   for(i=0;i<5;i=i+1)
   begin
   @(posedge I_CLK)   // 3rd data transfer
         begin 
         #tsu 
            I_IRDY_N = 1 ;              //  wait state
            AD[63:32] = 32'bz ;
            AD[31:0] = wr_data + 2; // keep previous data
            I_CBE_N[3:0] = 4'b0 ;
         end
   end      
   @(posedge I_CLK)
   begin
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data + 3;
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 0;  
      I_IRDY_N = 0 ;
      wait(!TRDY_N) ; // for trdy to be low
   end 
   @(posedge I_CLK)   // 4th data transfer not occurs
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
         #tsu 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[3:0] = 4'bz ;
         end
  end       
endtask 

/****************************/
// back to back
// write a burst data to the target  -- 32 bit data width
// increment data for each write 
// read the data back
task mem_back_back ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ;
   input [31:0] rd_addr ;
   input [3:0] burst_len ;  // for check Cn be a different #

   reg [3:0] length_count ;
   reg [31:0] data ;
   reg [31:0] data_recd ;
 
   begin 

   // address- command phase 

   @(posedge I_CLK)
   #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      length_count = burst_len ;
      data = wr_data ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_IRDY_N = 0 ;
      length_count = length_count -1 ;

      while (length_count > 1)
      begin 
         @(posedge I_CLK)
            if (!TRDY_N )  // one write has happened, give next data 
            begin 
            #tsu  data = data +1 ;
                  length_count = length_count -1 ;
                  AD[31:0] = data ;
            end 
      end  
   wait(!TRDY_N)
      $display("came 3");
   @(posedge I_CLK)  
      #tsu
         I_FRAME_N =  1 ; // indicating last transfer
         data =data + 1;
         AD[31:0] = data;

   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         #tsu 
            begin 
            I_FRAME_N = 0 ;
            I_IRDY_N = 1 ;
            AD[63:32] = 32'bz ;
            AD[31:0]  = rd_addr ;
            I_CBE_N[3:0] = 4'h6 ; // "0110" = mem_read command ; // 
            end

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N[3:0] = 4'b0 ;
      I_FRAME_N = 1; // indicating single word read 
      I_IRDY_N = 0 ;

     wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
            data_recd = IO_AD ;
         #tsu 
            I_IRDY_N = 1 ;
            I_CBE_N[3:0] = 4'bz ;
            $display ("Read address = %h received data = %h", rd_addr, data_recd);
         end
   end 
endtask 



/********************************************************************************/
//When a reserved burst command detected core will disconnect after 
//first data phase,by asserting TRDY_N and STOP_N   
task mem_rd_burst_res ;

   input [31:0] rd_addr ;
   input [3:0] burst_len ;

   reg [31:0] data_recd ;
   reg [3:0] length_count ;

   begin 

   // address/command phase 

   @(posedge I_CLK)
      #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = rd_addr ;
      I_CBE_N = 4'h6 ; // "0110" = mem_read command ; // 
      I_FRAME_N = 0 ;
      length_count = burst_len ;

   // data phase 

   @(posedge I_CLK)
      #tsu
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N = 4'b0 ;
      I_IRDY_N = 0 ;

   @(posedge I_CLK)
      #tsu
      I_IRDY_N = 1 ; //force wait cycles
   @(posedge I_CLK)
      #100
      I_IRDY_N = 0;
         
   wait(!TRDY_N) // for TRDY to be asserted 

   while (length_count > 1)
   begin
   @(posedge I_CLK)
      begin
         data_recd = IO_AD ;
         $display ("&&1 Read address = %h received data = %h", rd_addr, data_recd);
         length_count = length_count - 1;
         rd_addr = rd_addr + 4 ;
       end  
       if(TRDY_N && ~STOP_N)
       begin  
         $display(" frame deasserted");
         I_FRAME_N = 1'b1;
         $display(" Read address = %h length_count = %h",rd_addr,length_count);
        
       end  
   end 
   @(posedge I_CLK)
      I_IRDY_N =1'b1;  
   end 
endtask


/********************************************************************************/
// write a single word to the target  -- 32 bit data width
// command mem wr & invalidate
task mem_wr_inval ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ; 

  begin 

   // address/command phase 
   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N = 4'hf ; // "1111" = mem_wr & invalidate ;
      I_FRAME_N = 0 ;
   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N = 4'b0 ;
      I_FRAME_N = 1; // indicating single word write 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
         #tsu 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N = 4'bz ;
         end
   end 
endtask 


/*******************************************************************************/
//check the response for a reserved command 
task mem_rd_res_comm ;

   input [31:0] rd_addr ;
   input [3 :0] com_word;
   integer      i;
 
   reg [31:0] data_recd ;
   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = rd_addr ;
      I_CBE_N = com_word ;// write a reserved command word   // 
      I_FRAME_N = 0 ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N = 4'h0 ;
      I_FRAME_N = 1; // indicating single word read 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
            data_recd = IO_AD ;
         #tsu 
            I_IRDY_N = 1 ;
            I_CBE_N = 8'bz ;
            $display ("Read address = %h received data = %h", rd_addr, data_recd);
         end
  
//check for DEVSEL_n
    for(i=1;i<6;i=i+1)
    begin
	@(posedge I_CLK)
       	if(DEVSEL_N)
		  $display ("DEVSEL_N is not asserted");
        else
          $display ("ERROR occured"); 	
    end     
	@(posedge I_CLK)
       	if(DEVSEL_N)
		  $display ("Responding properly for reserved command");
        else
          $display ("ERROR occured"); 	

   end 
endtask 


/********************************************************************************/
// To check in valid byte enable, target abort should occure

task tgt_abort ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ; 
   input [ 3:0] byte;  
   integer i;
   
begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N = 4'h3 ; // "0011" = io_wr ;
      I_FRAME_N = 0 ;

   // data phase 
    
   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N = byte;//byte = 1110 and AD[1:0]="01"makes an invalid byte enable
      I_FRAME_N = 1; // indicating single word write 
      I_IRDY_N = 0 ;
//Following test shoulod be done for a few clock cycles      
   for(i=1 ;i<6 ; i = i+1)
   begin
   @(posedge I_CLK)
      if(DEVSEL_N & ~STOP_N )
         $display("Target is aborted ,Responding properly for invalid byte enables");
      else
         $display("Error occured in response to invalid byte enables");    
   end
end  
endtask


/***********************************************************************************/

// Test plan 2.1.1.3.7
// with out asserting IDSEL issue an config wr command and check for DEVSEL

task idsel_config_wr ;
   input [31:0] reg_addr ;
   input [31:0] data_to_write ;
   input [3 :0] byte ;   
   
   begin 
   @(posedge I_CLK)
   #tsu I_IDSEL = 0;
	  AD[63:32] = 32'bz ;
      AD[31:0]  = reg_addr ;
      I_CBE_N[3:0] = 4'hB ; //  command for config write 
      I_FRAME_N = 0 ;
   @(posedge I_CLK)  
   #tsu I_IDSEL = 0 ;
      I_CBE_N[3:0] = byte ;
	  AD[63:32] = 32'bz ;
      AD[31:0]  = data_to_write ;
      I_IRDY_N = 0 ;
      I_FRAME_N = 1 ;
      PAR = gen_prty(reg_addr,I_CBE_N) ;
	  $display ("At time %d writing %h at register address %h", $time,data_to_write,reg_addr) ;
   @(posedge I_CLK)
   #tsu 
      PAR = gen_prty(data_to_write,I_CBE_N) ;
   @(posedge TRDY_N)
      #tsu   I_IRDY_N = 1 ;
           AD = 64'bz ;
   
 end
endtask 

/****************************/
// write a burst data to the target  -- 32 bit data width
// check for serr signal when wrong parity is detected 

task mem_wr_serr ;
   input [31:0] wr_addr ;
   input [31:0] wr_data ;
   input [3:0] burst_len ;  // for check Cn be a different #
   integer i;
   
   reg [3:0] length_count ;
   reg [31:0] data ;
   reg PAR1;
   begin 

   // address- command phase 

   @(posedge I_CLK)
   #tsu 
	  AD[63:32] = 32'bz ;
      AD[31:0]  = wr_addr ;
      I_CBE_N[3:0] = 4'h7 ; // "0111" = mem_wr ;
      I_FRAME_N = 0 ;
      length_count = burst_len ;
      data = wr_data ;
      PAR1 = gen_prty(wr_addr,I_CBE_N);  

   // data phase 

   @(posedge I_CLK)
   #tsu
         
      AD[63:32] = 32'bz ;
      AD[31:0] = wr_data;
      I_CBE_N[3:0] = 4'b0 ;
      I_IRDY_N = 0 ;
      length_count = length_count -1 ;
      PAR = ~PAR1; 
      while (length_count > 1)
      begin 
      
         @(posedge I_CLK)
            if (!TRDY_N )  // one write has happened, give next data 
            begin 
            #tsu  
                  data = data +1 ;
                  length_count = length_count -1 ;
                  AD[31:0] = data ;
            end 
      
       end  
   wait(!TRDY_N)
   @(posedge I_CLK)  
      #tsu
         I_FRAME_N =  1 ; // indicating last transfer
         data =data + 1;
         AD[31:0] = data;
       

   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         #tsu 
            begin 
            I_IRDY_N = 1 ;
            AD[63:0] = 64'bz ;
            I_CBE_N[3:0] = 4'bz ;
        
    end
   end 
endtask


//read a single 32 bit word from memory 
task mem_rd_alias ;

   input [31:0] rd_addr ;
   input [3 :0] comm_word;
   
   reg [31:0] data_recd ;
   begin 

   // address/command phase 

   @(posedge I_CLK)
   #tsu  
	  AD[63:32] = 32'bz ;
      AD[31:0]  = rd_addr ;
      I_CBE_N = comm_word;//4'h6 ; // "0110" = mem_read command ; // 
      I_FRAME_N = 0 ;

   // data phase 

   @(posedge I_CLK)
   #tsu 
      AD[63:32] = 32'bz ;
      AD[31:0] = 32'bz ; // force a turnaound cycle 
      I_CBE_N = 4'b0 ;
      I_FRAME_N = 1; // indicating single word read 
      I_IRDY_N = 0 ;

      wait(!TRDY_N) ; // for trdy to be low
   @(posedge I_CLK) 
      if ( TRDY_N == 1'b0) // transfer should have taken place at this edge 
         begin 
            data_recd = IO_AD ;
         #tsu 
            I_IRDY_N = 1 ;
            I_CBE_N = 4'bz ;
            $display ("Read address = %h received data = %h", rd_addr, data_recd);
         end
   end 
endtask 

 