/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_wr_tb32.v
   Created on           : 19 May 
   Function             : 
   Author               : Pramod G
   Revision             : 
   Verified             : yes/no
   Modification History : 

   Instantiated in      : 

   Instantiates         :
=========================================================================*/
// to debug reads from the target
// test plan no: 2.1.1.2 - 3   mem_rd
// test plan no: 2.1.1.2 - 11  io_rd

module tgt_rd_tb ;

parameter half_pci_cycle = 15 ;
parameter half_app_cycle = 22 ; // arbitrary number

reg I_RST_N ;
reg I_CLK ;
reg I_FRAME_N ;
reg I_REQ64_N ;
reg [7:0] I_CBE_N ;////upper limit changed to 7 from 3
reg I_IDSEL ;
reg I_IRDY_N ;

reg I_app_clk ;
reg I_tgt_wr_nxt ;

reg I_capable64 ;
reg I_disc_from_app ;
reg I_abort_from_app ;
reg I_back_end_busy ;

reg [31:0] I_data_hi ;
reg [31:0] I_data_lo ;
reg I_data_valid ;

wire STOP_N ;
wire DEVSEL_N ;
wire TRDY_N ;
wire ACK64_N ;
wire [63:0] IO_AD ;  // from the chip
wire IO_PAR ;
wire IO_PAR64 ;

wire tgt_wr_data_avail ;
wire tgt_do_rd_xfer ;
wire [31:0] O_data_hi ;
wire [31:0] O_data_lo ;
wire [31:0] O_addr_hi ;
wire [31:0] O_addr_lo ;
wire O_data_valid_hi ;
wire O_data_valid_lo ;


reg [63:0] AD ; // for procedural assignment 
reg PAR ;
reg PAR64 ;

reg [31:0] read_data ;

assign IO_AD = AD ;

initial 
begin
#0 
   I_RST_N = 1 ;
   I_CLK  = 0;
   I_FRAME_N = 1 ;
   I_REQ64_N  = 1;
   I_CBE_N  = 0;
   I_IDSEL  = 0;
   I_IRDY_N  = 1;
   AD = 64'bz ;
   I_app_clk = 0 ;
   I_capable64  = 0 ;
   I_disc_from_app  = 0 ;
   I_abort_from_app  = 0 ;
   I_back_end_busy = 0  ;
   I_tgt_wr_nxt = 0 ;
   I_data_hi = 0 ;
   I_data_lo = 0 ;

#100 
   I_RST_N = 0 ;

#100 
   I_RST_N = 1; 

#100  // Enable memory space  and io space response in target device 

   config_wr (32'h04,32'h3) ; // h04 is the command status reg ; lower most 2 bits to be set
   config_rd (32'h04,read_data ) ;

/********** mem rd ************/
#100 
   set_mem_base_addr(32'h00FFF000);

#100
  $display (" Now do a single word read");
  mem_rd (32'h00FFF000) ;
  $display (" Done a single word write");

#1000 
   $display (" Now do a burst read");
   mem_rd_burst (32'h00FFF010,4'd2) ;
   $display (" Done a burst read");

#1000 
   $display (" Now do a burst read");
   mem_rd_burst (32'h00FFF010,4'd3) ;
   $display (" Done a burst read");
   
#1000 
   $display (" Now do a burst read");
   mem_rd_burst (32'h00FFF014,4'd8) ;
   $display (" Done a burst read");

/****** I/O rd *******/
#100 
   set_io_base_addr(32'h00FF0000);

#100
  $display (" Now do a single word read");
  io_rd (32'h00FF0000) ;
  $display (" Done a single word write");

#1000 
   $display (" Now do a burst read");
   io_rd_burst (32'h00FF0010,4'd2) ;
   $display (" Done a burst read");

#1000 
   $display (" Now do a burst read");
   io_rd_burst (32'h00FF0010,4'd3) ;
   $display (" Done a burst read");
   

#100 $stop;
end 


always 
begin
#half_pci_cycle  I_CLK = ~I_CLK ;
end

always 
begin
#half_app_cycle  I_app_clk = ~I_app_clk ;
end


always@(tgt_wr_data_avail)
begin
   if (tgt_wr_data_avail) 
      #2 I_tgt_wr_nxt = 1 ;
   else
      #2 I_tgt_wr_nxt = 0 ;
end     

// monitor the writes 
always@(posedge I_app_clk) 
begin 

if (O_data_valid_lo) 
   $display("Data = %h written at address  %h at t=%0d",
             O_data_lo, O_addr_lo ,$time ) ;
end


// transfer the requested reads 

always@(posedge I_app_clk)
begin

   if (tgt_do_rd_xfer)  // right now stick to incremental data,. later can read form file
      begin 
         #3 ;
         I_data_lo <= I_data_lo + 1 ;  
         I_data_valid <= 1'b1 ;
      end 
   else
      #2 
      I_data_valid <= 1'b0 ;
end 

tgt_top  dut (

						.I_RST_N (I_RST_N),
						.I_CLK    (I_CLK),
						.I_FRAME_N (I_FRAME_N),
						.I_REQ64_N (I_REQ64_N),
						.I_CBE_N   (I_CBE_N),
						.I_IDSEL   (I_IDSEL),
						.I_IRDY_N  (I_IRDY_N), 
	
                        .I_app_clk (I_app_clk),
                        .I_tgt_wr_nxt    (I_tgt_wr_nxt),					
						.I_capable64 (I_capable64),
						.I_disc_from_app (I_disc_from_app),
						.I_abort_from_app (I_abort_from_app),
						.I_back_end_busy  (I_back_end_busy),
			
                        .I_data_hi    (I_data_hi),
                        .I_data_lo    (I_data_lo),
                        .I_data_valid (I_data_valid),

						.O_STOP_N     (STOP_N),
						.O_TRDY_N     (TRDY_N),
						.O_DEVSEL_N   (DEVSEL_N),
						.O_ACK64_N    (ACK64_N),
					
						.IO_AD        (IO_AD ),
                        .IO_PAR       (IO_PAR),
                        .IO_PAR64     (IO_PAR64),

                        .O_tgt_wr_data_avail (tgt_wr_data_avail),
                        .O_tgt_do_rd_xfer    (tgt_do_rd_xfer),  
                        .O_data_lo           (O_data_lo),        
                        .O_data_hi           (O_data_hi), 
                        .O_addr_lo           (O_addr_lo),        
                        .O_addr_hi           (O_addr_hi),
                        .O_data_valid_lo     (O_data_valid_lo),
                        .O_data_valid_hi     (O_data_valid_hi)



);

`include "tasks.v"

endmodule
