/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_wr_tb64.v
   Created on           : 20 May 
   Function             : 
   Author               : Pramod G
   Revision             : 
   Verified             : yes/no
   Modification History : 

   Instantiated in      : 

   Instantiates         :
=========================================================================*/
// to check 64 bit writes into the target with 32 bit addressing
// to check 64 bit writes into the target with 64 bit addressing
// to check 32 bit writes into the target with 64 bit addressing
// test plan no: 2.1.1.2 - 4
// test plan no: 2.1.1.2 - 6
// test plan no: 2.1.1.2 - 8

module tgt_wr_tb ;

parameter half_pci_cycle = 15 ;
parameter half_app_cycle = 22 ; // arbitrary number

reg I_RST_N ;
reg I_CLK ;
reg I_FRAME_N ;
reg I_REQ64_N ;
reg [7:0] I_CBE_N ;
reg I_IDSEL ;
reg I_IRDY_N ;

reg I_app_clk ;
reg I_tgt_wr_nxt ;

reg I_capable64 ;
reg I_disc_from_app ;
reg I_abort_from_app ;
reg I_back_end_busy ;

reg [31:0] I_data_hi ;
reg [31:0] I_data_lo ;
reg I_data_valid ;

wire STOP_N ;
wire DEVSEL_N ;
wire TRDY_N ;
wire ACK64_N ;
wire [63:0] IO_AD ;  // from the chip
wire IO_PAR ;
wire IO_PAR64 ;

wire tgt_wr_data_avail ;
wire tgt_do_rd_xfer ;
wire [31:0] O_data_hi ;
wire [31:0] O_data_lo ;
wire [31:0] O_addr_hi ;
wire [31:0] O_addr_lo ;
wire O_data_valid_hi ;
wire O_data_valid_lo ;


reg [63:0] AD ; // for procedural assignment 
reg PAR ;
reg PAR64 ;

reg [31:0] read_data ;

assign IO_AD = AD ;
assign IO_PAR = PAR ;
assign IO_PAR64 = PAR64 ;

initial 
begin
#0 
   I_RST_N = 1 ;
   I_CLK  = 0;
   I_FRAME_N = 1 ;
   I_REQ64_N  = 1;  
   I_CBE_N  = 8'b0;
   I_IDSEL  = 0;
   I_IRDY_N  = 1;
   AD = 64'bz ;
   PAR = 1'bz ;  // should modify for parity checks later
   PAR64 = 1'bz ;
   I_app_clk = 0 ;
   I_capable64  = 1 ; // asserted for 64 bit 
   I_disc_from_app  = 0 ;
   I_abort_from_app  = 0 ;
   I_back_end_busy = 0  ;
   I_tgt_wr_nxt = 0 ;
   I_data_hi = 0 ;
   I_data_lo = 0 ;
   I_data_valid = 0;
#100 
   I_RST_N = 0 ;

#100 
   I_RST_N = 1; 

#100  // Enable memory space  and io space response in target device 

   config_wr (32'h04,32'h3) ; // h04 is the command status reg ; lower most 2 bits to be set
   config_rd (32'h04,read_data ) ;
#100 
   set_mem_base_addr(32'h00FFF000);

#100 
   
   $display (" Now do a single word write - 64 addr");
   mem_wr_64_addr (64'h00FFF00000FFF000,32'hFFFFFFFF) ;
   $display (" Done a single word write - 64 addr");
   
#1000 
   $display (" Now do a single word burst write - 64 addr");
   mem_wr_64_addr_burst (64'h00FFF11100FFF111,32'h0000FFFF,4'h4) ;
   $display (" Done a single word burst write -- 64 addr");

#1000 
   $display (" Now do a double word write - 64 addr");
   mem_wr_64 (64'h00FFF22200FFF222,64'h11111111FFFFFFFF) ;
   $display (" Done a double word write - 64 addr");

#1000 
   $display (" Now do a double word burst write - 64 addr");
   mem_wr_64_burst (64'h00FFF33300FFF333,64'h11112222FFFFFFFF,4'h3) ;
   $display (" Done a double word burst write - 64 addr");

#1000 
   $display (" Now do a double word write - 32 addr");
   mem_wr_64_dat (32'h00FFF444,64'h22221111FFFFFFFF) ;
   $display (" Done a double word write - 32 addr");
#1000 
   $display (" Now do a double word burst write - 32 addr");
   mem_wr_64_dat_burst (32'h00FFF555,64'h22222222FFFFFFFF,4'h3) ;
   $display (" Done a double word burst write - 32 addr");

  
#100 $stop;
end 

always @(ACK64_N)
begin
  $display("ACK64_N=%d",ACK64_N);
end

always 
begin
#half_pci_cycle  I_CLK = ~I_CLK ;
end

always 
begin
#half_app_cycle  I_app_clk = ~I_app_clk ;
end


always@(tgt_wr_data_avail)
begin
   if (tgt_wr_data_avail) 
      #2 I_tgt_wr_nxt = 1 ;
   else
      #2 I_tgt_wr_nxt = 0 ;
end     

// monitor the writes  that happen at the fifo to 
// application interface
always@(posedge I_app_clk) 
begin 
   if (O_data_valid_lo) 
      $display("Data lo = %h written at address lo %h at t=%0d",
               O_data_lo, O_addr_lo ,$time ) ;             
   if (O_data_valid_hi) 
   $display("Data hi= %h written at address hi  %h at t=%0d",
             O_data_hi, O_addr_hi ,$time ) ;            

end

tgt_top  dut (

						.I_RST_N (I_RST_N),
						.I_CLK    (I_CLK),
						.I_FRAME_N (I_FRAME_N),
						.I_REQ64_N (I_REQ64_N),
						.I_CBE_N   (I_CBE_N),
						.I_IDSEL   (I_IDSEL),
						.I_IRDY_N  (I_IRDY_N), 
	
                        .I_app_clk (I_app_clk),
                        .I_tgt_wr_nxt    (I_tgt_wr_nxt),					
						.I_capable64 (I_capable64),
						.I_disc_from_app (I_disc_from_app),
						.I_abort_from_app (I_abort_from_app),
						.I_back_end_busy  (I_back_end_busy),
			
                        .I_data_hi    (I_data_hi),
                        .I_data_lo    (I_data_lo),
                        .I_data_valid (I_data_valid),

						.O_STOP_N     (STOP_N),
						.O_TRDY_N     (TRDY_N),
						.O_DEVSEL_N   (DEVSEL_N),
						.O_ACK64_N    (ACK64_N),
					
						.IO_AD        (IO_AD ),
						.IO_PAR       (IO_PAR ),
						.IO_PAR64     (IO_PAR64 ),

                        .O_tgt_wr_data_avail (tgt_wr_data_avail),
                        .O_tgt_do_rd_xfer    (tgt_do_rd_xfer),  
                        .O_data_lo           (O_data_lo),        
                        .O_data_hi           (O_data_hi), 
                        .O_addr_lo           (O_addr_lo),        
                        .O_addr_hi           (O_addr_hi),
                        .O_data_valid_lo     (O_data_valid_lo),
                        .O_data_valid_hi     (O_data_valid_hi)



);

`include "tasks.v"

endmodule
