/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : addr_incr.v
   Created on           : 19 Feb 1999
   Function             : Incrementor for target address  
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 
   26 March               dual address cycle is sufficient condition for 
                          loading the 64 bit address. Second xfer cycle
                          signal removed from the ports 
   Instantiated in      : tgt_addr_ctl.v

   Instantiates         :
   Synthesis check      : not done 
=========================================================================*/

module addr_incr ( 
                     I_RST_N,
                     I_CLK,              // The clock to this module comes from app side
                     first_xfer_cycle,   // inputs should be  synchronised to this clock
                     dual_addr_cycle,
                     incr_addr,
                     load_addr, 
                     out_addr
) ;

parameter addr_width = 64 ;

input I_CLK ;
input I_RST_N ;
input first_xfer_cycle;
input dual_addr_cycle;
input incr_addr ;
input [addr_width-1 :0 ] load_addr ;
output [addr_width-1 :0 ] out_addr ;
   
reg [addr_width-1 : 0] out_addr ;

reg load_32bit_addr ;
reg load_64bit_addr ;

reg bar32_64 ; // when 0 - indicates 32 bit address ; when 1 indicates 64 bit address


// Generate the load enable for the address counter.
// At the first cycle, dual address cycle = 0, indicates 32 bit addressing

always@(posedge I_CLK or negedge I_RST_N)
begin

if (I_RST_N == 1'b0)
   begin 
      load_32bit_addr <= 1'b0 ;
      load_64bit_addr <= 1'b0 ;
      bar32_64 <= 1'b0 ;
   end
else
   begin 

      if (first_xfer_cycle == 1'b1) 
         begin

         if (dual_addr_cycle == 1'b0) // for 64 bit xfers, comes along with first_xfer_cyc
            begin 
               load_32bit_addr <= 1'b1 ;
               bar32_64 <= 1'b0 ;
            end
         else // (dual_addr_cycle == 1'b1)
            begin 
               load_64bit_addr <= 1'b1 ;
               bar32_64 <= 1'b1 ;
            end
         end
      else
         begin
            load_32bit_addr <= 1'b0 ;
            load_64bit_addr <= 1'b0 ;
         end 
   end 
end 


// Probably this count process will have to be modified. At this stage,
// I'm not sure if this will synthesise and meet the timings for 66 
// MHz operation

always@(posedge I_CLK or negedge I_RST_N)
begin

if (I_RST_N == 1'b0)

   out_addr <= 0  ;

else
   begin
   if (load_32bit_addr == 1'b1) out_addr <= load_addr[31:0] ;
   
   if ((load_64bit_addr == 1'b1) & (addr_width == 64))  out_addr <= load_addr[63:0] ;
  
   if ((incr_addr ==1'b1) && (bar32_64 ==1'b0))  out_addr <= out_addr + 4 ;

   if ((incr_addr ==1'b1) && (bar32_64 ==1'b1)) out_addr <= out_addr + 8 ; 
   end 
end 
endmodule




