/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : cmd_reg.v
   Created on           : 17 Feb 1999
   Function             : Models command register at address 04h;low 2 bytes
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 
   4th Mar 99             Naming conventions followed
   Instantiated in      : sts_cmd_reg.v

   Instantiates         : reg_bit_0.v
   Synthesis check      : not done 
=========================================================================*/

/* This is the command register at address 04h ; 
 Contains the lower two bytes at that address ;
 The higher bytes are occupied by status register

 The folowing are the bits implemented 

 bit 0 : R/W  When 1 enables I/O address decoder     ; default 0 
 bit 1 : R/W  When 1 enables memory address decoder  ; default 0 
 bit 2 : R/W  When 1 enables bus mastering           ; default 0 
 bit 3 : R/O  Special cycle recognition not available; default 0 
 bit 4 : R/O  Memory write & invalidate not available; default 0 
 bit 5 : R/O  VGA palette snoop not available        ; default 0 
 bit 6 : R/W  When 1 device can report parity errors ; default 0 
 bit 7 : R/O  Core does not do data/address stepping ; default 0
 bit 8 : R/W  When 1 enables SERR generation         ; default 0 
              ( Address parity err & bit 6 = 1 & bit 8 = 1 generates SERR
 bit 9 : R/O  Core does not do fast back to back     ; default 0 
              transfers in initiator mode 

 bits 10-15 : reserved ; return all 0's when read 

*/

module cmd_reg (
                    I_CLK,
                    I_RST,
                    I_in_data,
                    reg_sel,
                    I_wr_enb,
                    I_rd_enb,
                    O_out_data,
                    O_io_spc_enb,     
                    O_mem_spc_enb,  
                    O_prty_err_resp, 
                    O_master_enb,    
                    O_serr_enb     
);
input [15:0] 	I_in_data ;
input 		I_CLK ;
input 		I_RST ;
input 		reg_sel ;
input 		I_wr_enb ;
input 		I_rd_enb ;

output [15:0] 	O_out_data ; 
output O_io_spc_enb ;    
output O_mem_spc_enb;  
output O_prty_err_resp; 
output O_master_enb;    
output O_serr_enb;     

wire [15:0] cmd_reg_val ;
wire 	wr_enab ;
wire	rd_enab ;

assign wr_enab = I_wr_enb & reg_sel ;

assign O_out_data = cmd_reg_val ;

// Register bits going to the core

assign O_io_spc_enb = cmd_reg_val[0] ;
assign O_mem_spc_enb = cmd_reg_val[1] ;
assign O_master_enb = cmd_reg_val[2] ;
assign O_prty_err_resp = cmd_reg_val[6] ;
assign O_serr_enb = cmd_reg_val[6] ;

// writable registers 
reg_bit_0 bit_0 (I_RST, I_in_data[0], I_CLK, wr_enab, cmd_reg_val [0] ); 
reg_bit_0 bit_1 (I_RST, I_in_data[1], I_CLK, wr_enab, cmd_reg_val [1] ); 
reg_bit_0 bit_2 (I_RST, I_in_data[2], I_CLK, wr_enab, cmd_reg_val [2] ); 
reg_bit_0 bit_6 (I_RST, I_in_data[6], I_CLK, wr_enab, cmd_reg_val [6] ); 
reg_bit_0 bit_8 (I_RST, I_in_data[8], I_CLK, wr_enab, cmd_reg_val [8] ); 

assign cmd_reg_val [5:3] = 3'b0 ;
assign cmd_reg_val [7] = 1'b0 ;
// No VGA palette snoop enable; No memory write and invalidate ; No special
//cycle monitoring capability ; No address data stepping as master

assign cmd_reg_val [15:9] = 7'b0 ;
// 15:8 = reserved Bit 9 : Back to back enable  - set to 0

endmodule 
