/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : par_check.v
   Created on           : 19 March 99
   Function             : Parity check for target writes. This block checks
                          for only data parity errors. Address parity errors
                          are genarated in the addr_latch block. However the
                          final parity error signal is coming out of this 
                          block  
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   Instantiated in      : tgt_top.v 

   Instantiates         :
   Synthesis check      : done/not done 
=========================================================================*/

module par_check (


   I_RST_N,
   I_CLK,
   I_incr_twf_wrptr, // ct each clock edge with this high, latch the parity
   I_par_from_pci,
   I_par64_from_pci,
   I_ack64_N,
   I_twf_data_hi,
   I_twf_data_lo,
   I_prty_err_resp,

   O_set_detected_prty_err,     // should be set even if prty_err_resp bit is 0
   O_perr

);


input I_RST_N ;
input I_CLK ;
input I_incr_twf_wrptr ;
input I_par_from_pci ;
input I_par64_from_pci ;
inout I_ack64_N ;
input [31:0] I_twf_data_hi ;
input [31:0] I_twf_data_lo ;
inout I_prty_err_resp ;

output O_set_detected_prty_err ;
output O_perr ;

reg calcd_par ;
reg calcd_par64 ; 
reg latched_par ;
reg latched_par64 ;
reg par_err ;

// parity latch 

always@(negedge I_RST_N or posedge I_CLK)
begin

if (!I_RST_N) 
   begin 
      latched_par <= 1'b0 ;
      latched_par64 <= 1'b0 ;
   end
else
   begin
   // do this at all clock edges 
      latched_par <= I_par_from_pci ;     
      latched_par64 <= I_par64_from_pci ;     
   end 
end

// parity calc and compare 


always@(negedge I_RST_N or posedge I_CLK)
begin

if (!I_RST_N) 
   begin 
      calcd_par = 1'b0 ;
      calcd_par64 = 1'b0 ;
   end
else
   begin
   if (I_incr_twf_wrptr)
      begin
         calcd_par = calc_prty(I_twf_data_lo) ;
         calcd_par64 = calc_prty(I_twf_data_hi) ;

      end 
   // do this at all clock edges  // 
 
   if ((calcd_par != latched_par)|| (calcd_par64 != latched_par64 ) ) 
 // only data parity error now
      par_err <= 1'b1;
   else
      par_err <= 1'b0 ;
   end
end


assign O_set_detected_prty_err = par_err ;

function calc_prty ;
input [31:0] in_data ; 
begin

   calc_prty = ^in_data ;

end 
endfunction


endmodule 
