/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : regout_mux.v
   Created on           : 4th March 99
   Function             : Multiplexes the register output data 
   Author               : Ramaprasad K V
   Revision             : 
   Verified             : yes/no
   Modification History : 

   Instantiated in      : cfg_space.v

   Instantiates         :
   Synthesis check      : done/not done 
=========================================================================*/

module regout_mux (

					I_device_reg_out  ,
					I_cmd_sts_reg_out  ,
                    I_class_code_reg_out ,
					I_lat_timer_reg_out  ,
					I_bar0_lo_reg_out  ,
					I_bar0_hi_reg_out  ,
					I_bar1_reg_out  ,
					I_intr_reg_out ,
					I_rd_enb,

                    I_device_reg_sel,  			// device register
                    I_cs_reg_sel,				// command-status register
                    I_cc_reg_sel,				// class code register
                    I_lt_reg_sel,			// latency timer register
                    I_bar0_lo_reg_sel,
                    I_bar0_hi_reg_sel,
                    I_bar1_reg_sel,
	 				I_intr_reg_sel,
					O_cfg_regs_data 


);


input [31:0] I_device_reg_out ;
input [31:0] I_cmd_sts_reg_out ;
input [31:0] I_class_code_reg_out ;
input [31:0] I_lat_timer_reg_out ;
input [31:0] I_bar0_lo_reg_out ;
input [31:0] I_bar0_hi_reg_out ;
input [31:0] I_bar1_reg_out ;
input [31:0] I_intr_reg_out ;

input I_rd_enb ;                

input I_device_reg_sel ;         
input I_cs_reg_sel ;           		
input I_cc_reg_sel ;        	
input I_lt_reg_sel ;         	
input I_bar0_lo_reg_sel ;      
input I_bar0_hi_reg_sel ;      
input I_bar1_reg_sel ;         
input I_intr_reg_sel ;

output	[31:0] O_cfg_regs_data ; // valid_cfg_rd can be used as out_enbable 

reg	[31:0] O_cfg_regs_data ; 
wire [7:0] all_reg_sels ;

// easy to maniputale with one vector !! 

assign all_reg_sels = {I_device_reg_sel, I_cs_reg_sel, I_cc_reg_sel,
                       I_lt_reg_sel, I_bar0_lo_reg_sel, I_bar0_hi_reg_sel,
                       I_bar1_reg_sel, I_intr_reg_sel } ;

always@(I_device_reg_out or I_cmd_sts_reg_out or I_class_code_reg_out or 
        I_lat_timer_reg_out or I_bar0_lo_reg_out or I_bar0_hi_reg_out or 
	    I_bar1_reg_out or I_intr_reg_out or I_rd_enb or all_reg_sels )
begin 

if (I_rd_enb)

   case (all_reg_sels)

   8'b10000000 :  O_cfg_regs_data  <= I_device_reg_out ;

   8'b01000000 :  O_cfg_regs_data  <= I_cmd_sts_reg_out ;

   8'b00100000 :  O_cfg_regs_data  <= I_class_code_reg_out ;

   8'b00010000 :  O_cfg_regs_data  <= I_lat_timer_reg_out ;

   8'b00001000 :  O_cfg_regs_data  <= I_bar0_lo_reg_out ;

   8'b00000100 :  O_cfg_regs_data  <= I_bar0_hi_reg_out ;

   8'b00000010 :  O_cfg_regs_data  <= I_bar1_reg_out ;

   8'b00000001 :  O_cfg_regs_data  <= I_intr_reg_out ;

   default     :  O_cfg_regs_data  <= 32'b0 ;

   endcase 

else
   O_cfg_regs_data <= 32'b0 ; // at this time outputs should not be driven at the pin level

end 

endmodule 
