/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_engine.v
   Created on           : 3 March 1999
   Function             : Target engine module 
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   12 March               O_read_mode port added
   15 Mar 99              I_do_disc port added 
   Instantiated in      : 

   Instantiates         : tgt_sm.v , tgt_misc.v , devsel_gen.v
   Synthesis check      : not done 
=========================================================================*/

module tgt_engine (


				I_RST_N,
				I_CLK,
                I_FRAME_N,			// direct from the PCI bus
                I_frame_N,	  		// latched version from tgt_addr_ctl
 				I_REQ64_N ,
				I_IRDY_N,
		 		I_capable64 ,

				I_disc_from_app,
				I_abort_from_app,
			
				I_first_xfer_cyc,		
				I_second_xfer_cyc,		  
                I_single_cyc_xfer,

                I_lo_addr_match,
               	I_hi_addr_match,
                I_io_addr_match,
                I_do_disc,
 
				I_io_rd,				// From Target command decoder
                I_io_wr,				
                I_mem_rd,			
                I_mem_wr,		
                I_dac,         
				I_valid_cfg_rd ,
				I_valid_cfg_wr ,
	
				I_io_spc_enb,
				I_mem_spc_enb,
			
				I_back_end_busy,

				I_twf_full,
				I_trf_empty,

                O_read_mode,
                O_end_xfer,
				O_stop_N ,
				O_oe_stop ,
				O_devsel_N ,
				O_oe_devsel ,
				O_trdy_N ,
				O_oe_trdy ,
				O_ack64_N ,
				O_oe_ack64 ,
				O_oe_AD_lo,
				O_oe_AD_hi,
                O_oe_PAR ,
                O_oe_PAR64, 	
				O_incr_twf_wrptr,
				O_incr_trf_rdptr


) ;



input		I_RST_N;
input		I_CLK;
input       I_FRAME_N;
input       I_frame_N;
input       I_IRDY_N;
input 		I_REQ64_N ;
input 		I_capable64 ;

input		I_disc_from_app ;
input 		I_abort_from_app ;

input 		I_twf_full ;
input 		I_trf_empty ;

input		I_first_xfer_cyc;		
input		I_second_xfer_cyc;		
input		I_single_cyc_xfer;		

input		I_io_rd;	
input       I_io_wr;
input       I_mem_rd;			
input       I_mem_wr;		
input       I_dac; 

input		I_valid_cfg_rd ;
input		I_valid_cfg_wr ;

input 		I_lo_addr_match ; 
input 		I_hi_addr_match ; 
input 		I_io_addr_match ;
input 		I_do_disc ;

input		I_io_spc_enb ;
input		I_mem_spc_enb ;

input		I_back_end_busy ;

output      O_read_mode ;				
output      O_end_xfer ;				
output		O_oe_AD_lo ;
output		O_oe_AD_hi ;
output  	O_oe_PAR ;	
output  	O_oe_PAR64; 	
output		O_incr_twf_wrptr	;
output		O_incr_trf_rdptr	;

output		O_stop_N ;
output		O_trdy_N ;
output		O_devsel_N ;
output		O_ack64_N ;
output		O_oe_devsel ;
output		O_oe_stop ;
output		O_oe_trdy ;
output		O_oe_ack64 ;

wire 		tgt_rd ;
wire 		tgt_wr ;
wire		devsel_for_rdwr ; // i.e. exept config
wire		oe_devsel_rdwr ;
wire		devsel_for_cfg ;
wire		oe_devsel_cfg ;

wire		new_tgt_cmd ;
wire		last_xfer_flag ;
wire 		end_xfer ;
wire		stop_1 ;
wire		stop_2 ;
wire		stop_3 ;
wire		trdy_1 ;
wire		trdy_2 ;
wire		trdy_3 ;
wire		oe_stop_1 ;
wire		oe_stop_2 ;
wire		oe_stop_3 ;
wire		oe_trdy_1 ;
wire		oe_trdy_2 ;
wire		oe_trdy_3 ;
wire        trdy_N ; // internal signal 

wire        latched_irdy ;
wire        last_cyc_indi ;

assign	O_devsel_N 	= devsel_for_rdwr & devsel_for_cfg ; // 0 when any one is 0
assign  O_oe_devsel = oe_devsel_rdwr | oe_devsel_cfg ;


assign	trdy_N = trdy_1 & trdy_2 & trdy_3;    
assign  O_trdy_N = trdy_N ;

// if any one is 0, trdy should be 0 

assign	O_oe_trdy = oe_trdy_1 | oe_trdy_2 | oe_trdy_3  ; 
// if any one is 1, output should be driven

assign	O_stop_N = stop_1 & stop_2 & stop_3 ;
assign	O_oe_stop = oe_stop_1 | oe_stop_2 | oe_stop_3 ;

//assign O_end_xfer = end_xfer ; // use to reset FIFO pointers 
assign O_end_xfer = last_cyc_indi ; // use to reset FIFO pointers 

//devsel_gen devsel_gen (
devsel_sm devsel_sm (

					.I_RST_N		(I_RST_N),
					.I_CLK			(I_CLK),
					.I_REQ64_N		(I_REQ64_N),	
					.I_capable64	(I_capable64),	// when 1, backend has 64 bit data path

					.I_first_xfer_cyc	(I_first_xfer_cyc),	
					.I_second_xfer_cyc	(I_second_xfer_cyc),
//					.I_frame_N			(I_frame_N),
					.I_frame_N			(I_FRAME_N),  // EXPT
            
					.I_io_rd		(I_io_rd),		
                    .I_io_wr		(I_io_wr),		
                    .I_mem_rd		(I_mem_rd),		
                    .I_mem_wr		(I_mem_wr),		
                    .I_dac			(I_dac), 
					
                    .I_lo_addr_match	(I_lo_addr_match),
                    .I_hi_addr_match	(I_hi_addr_match),
                    .I_io_addr_match	(I_io_addr_match),

					.I_mem_spc_enb		(I_mem_spc_enb),			
					.I_io_spc_enb		(I_io_spc_enb),

//					.I_end_xfer			(last_xfer_flag),
					.I_end_xfer			(end_xfer),
					.I_back_end_busy	(I_back_end_busy),
					
					.O_devsel_N			(devsel_for_rdwr),
					.O_ack64_N			(O_ack64_N),
					.O_oe_devsel		(oe_devsel_rdwr) ,
					.O_oe_ack64			(O_oe_ack64),
					.O_stop_N			(stop_2),
					.O_oe_stop		    (oe_stop_2),
                    .O_trdy_N			(trdy_2),
					.O_oe_trdy			(oe_trdy_2),
                    .O_tgt_rd			(tgt_rd),
                    .O_tgt_wr			(tgt_wr),
					.O_new_tgt_cmd		(new_tgt_cmd)
);

tgt_sm tgt_sm (

				.I_RST_N			(I_RST_N),
				.I_CLK				(I_CLK),

				.I_devsel_N			(devsel_for_rdwr), 
//                .I_frame_N			(I_frame_N),	 
                .I_frame_N			(I_FRAME_N),	 
                .I_new_tgt_cmd		(new_tgt_cmd),
				.I_tgt_wr			(tgt_wr),
				.I_tgt_rd			(tgt_rd),

				.I_disc_from_app	(I_disc_from_app),	
				.I_abort_from_app	(I_abort_from_app),
                .I_do_disc          (I_do_disc),
			
				.I_irdy_N			(latched_irdy),

				.I_twf_full			(I_twf_full),	
				.I_last_xfer_flag 	(last_xfer_flag),
				.I_single_cyc_xfer 	(I_single_cyc_xfer),
				.I_trf_empty		(I_trf_empty),
			
				.O_trdy_N			(trdy_1),	
				.O_oe_trdy			(oe_trdy_1),
				.O_stop_N			(stop_1),
                .O_oe_stop			(oe_stop_1),

                .O_read_mode        (O_read_mode),
				.O_oe_AD_lo  		(O_oe_AD_lo),
				.O_oe_AD_hi  		(O_oe_AD_hi),
                .O_oe_PAR 			(O_oe_PAR),
                .O_oe_PAR64 		(O_oe_PAR64),
				.O_incr_twf_wrptr	(O_incr_twf_wrptr),
				.O_incr_trf_rdptr	(O_incr_trf_rdptr),
				.O_end_xfer			(end_xfer),
				.O_last_cyc_indi	(last_cyc_indi)
				
); 

tgt_misc tgt_misc ( 
					.I_RST_N		(I_RST_N),
					.I_CLK			(I_CLK),
					.I_FRAME_N		(I_FRAME_N),  // check
					.I_IRDY_N		(I_IRDY_N),
					.I_trdy_N		(trdy_N),
                    .O_latched_irdy (latched_irdy),
					.O_last_xfer_flag (last_xfer_flag)
);

cfg_sm cfg_sm (

					.I_RST_N        (I_RST_N),
					.I_CLK          (I_CLK),

					.I_first_xfer_cyc ( I_first_xfer_cyc),	
//					.I_frame_N      (I_frame_N),		
					.I_frame_N      (I_FRAME_N),		
                    .I_irdy_N       (latched_irdy),
    
					.I_valid_cfg_rd (I_valid_cfg_rd),	
					.I_valid_cfg_wr (I_valid_cfg_wr),	

					.O_devsel_N     (devsel_for_cfg),
					.O_oe_devsel    (oe_devsel_cfg), 	
					.O_stop_N       (stop_3),		
					.O_oe_stop      (oe_stop_3),	
                    .O_trdy_N       (trdy_3),
					.O_oe_trdy      (oe_trdy_3) 
);


endmodule
