/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_fifo.v 
   Created on           : 10 Mar 99
   Function             : Target Fifo
   Author               : Ramaprasad K V
   Revision             : 1.0
   Verified             : no
   Modification History : 

   Instantiated in      : 

   Instantiates         :
   Synthesis check      : done/not done 
=========================================================================*/

// This module instantiates the pci_fifo module ans creates a wrapper 
// around it for use in both target read and write modes 


module tgt_fifo
(
// inputs from PCI side 

   I_RST_N,
   I_CLK,
   I_fifo_reset, // use only for the read fifo
   I_twf_data_lo,
   I_twf_data_hi,
   I_incr_twf_wrptr,
   I_incr_trf_rdptr,

// inputs from app side 

   I_app_clk,
   I_tgt_wr_nxt,
   I_data_lo,
   I_data_hi,
   I_data_valid,

// outputs to PCI side 
   O_trf_data_lo,
   O_trf_data_hi,
   O_trf_empty,
   O_twf_full,

// outputs to app side 

   O_tgt_wr_data_avail,  
   O_trf_not_full,
   O_data_lo,
   O_data_hi
    
);

input   I_CLK ;
input   I_RST_N ;
input   I_fifo_reset ;
input [31:0] I_twf_data_lo ;
input [31:0] I_twf_data_hi ;
input   I_incr_twf_wrptr ;
input   I_incr_trf_rdptr ;

input   I_app_clk ;
input   I_tgt_wr_nxt ;
input [31:0] I_data_lo;
input [31:0] I_data_hi;
input   I_data_valid ;

output [31:0] O_trf_data_lo ;
output [31:0] O_trf_data_hi ;
output O_twf_full ;
output O_trf_empty ;


output  O_tgt_wr_data_avail;
output  O_trf_not_full ;
output [31:0] O_data_lo ;
output [31:0] O_data_hi ;

wire    twf_full ;
wire    twf_empty ;
wire    trf_full ;
wire    trf_empty ;

assign O_twf_full = twf_full ;
assign O_tgt_wr_data_avail = ~twf_empty ;

assign O_trf_not_full = ~trf_full ;
assign O_trf_empty = trf_empty ;

pci_fifo twf_low_bank
   (
    .I_data_in       (I_twf_data_lo),
    .I_we            (I_incr_twf_wrptr),
    .I_re            (I_tgt_wr_nxt),
    .I_pwrclr_N      (I_RST_N),
    .I_fifo_reset    (1'b0), 
    .O_full          (twf_full) ,
    .O_empty         (twf_empty),
    .O_data_out      (O_data_lo),
    .I_clk_wr        (I_CLK),
    .I_clk_rd        (I_app_clk)
    );

pci_fifo twf_hi_bank
   (
    .I_data_in       (I_twf_data_hi),  
    .I_we            (I_incr_twf_wrptr),
    .I_re            (I_tgt_wr_nxt),
    .I_pwrclr_N      (I_RST_N),
    .I_fifo_reset    (1'b0), 
    .O_full          (hi_twf_full), // this need not be used
    .O_empty         (hi_twf_empty), // for any decision logic
    .O_data_out      (O_data_hi),
    .I_clk_wr        (I_CLK),
    .I_clk_rd        (I_app_clk)
    );


pci_fifo trf_low_bank
   (
    .I_data_in       (I_data_lo),
    .I_we            (I_data_valid),
    .I_re            (I_incr_trf_rdptr),
    .I_pwrclr_N      (I_RST_N),
    .I_fifo_reset    (I_fifo_reset), 
    .O_full          (trf_full),
    .O_empty         (O_trf_empty),
    .O_data_out      (O_trf_data_lo),
    .I_clk_wr        (I_app_clk),
    .I_clk_rd        (I_CLK)
    );

pci_fifo trf_hi_bank
   (
    .I_data_in       (I_data_hi),
    .I_we            (I_data_valid),
    .I_re            (I_incr_trf_rdptr),
    .I_pwrclr_N      (I_RST_N),
    .I_fifo_reset    (I_fifo_reset), 
    .O_full          (hi_trf_full),
    .O_empty         (hi_trf_empty),
    .O_data_out      (O_trf_data_hi),
    .I_clk_wr        (I_app_clk),
    .I_clk_rd        (I_CLK)
    );


endmodule 
