/*=========================================================================
   CLPCI core                                        Chiplogic confidential
   
   File name            : tgt_rd_dp.v
   Created on           : 8th March 99
   Function             : Target read data path -- add parity function later
   Author               : Ramaprasad K V
   Revision             : 
   Verified             : no
   Modification History : 

   Instantiated in      : 

   Instantiates         :
   Synthesis check      : not done 
=========================================================================*/


module tgt_rd_dp (

					I_RST_N,
					I_CLK,

                    I_in_data_lo,
                    I_in_data_hi,
                    I_devsel_N ,

                    O_latched_data_lo,
                    O_latched_data_hi,
					O_par,
					O_par64


);

input    I_RST_N ;
input    I_CLK ;

input [63:32]   I_in_data_hi ;
input [31:0]    I_in_data_lo ;

input I_devsel_N ;

output O_par ;
output O_par64 ;

output [63:32]   O_latched_data_hi ;
output [31:0]    O_latched_data_lo ;

reg [63:32]   latched_data_hi ;
reg [31:0]    latched_data_lo ;
reg O_par ;
reg O_par64 ;

assign O_latched_data_hi = latched_data_hi ;
assign O_latched_data_lo = latched_data_lo ;

always@(posedge I_CLK or negedge I_RST_N)
begin

if (!I_RST_N)
   begin

      latched_data_hi <= 32'b0 ;
      latched_data_lo <= 32'b0 ;
      O_par <= 1'b0 ;
      O_par64 <= 1'b0 ;
   end 
else
   if (!I_devsel_N)  // when the target is selected 
      begin

         latched_data_hi <= I_in_data_hi ; 
         latched_data_lo <= I_in_data_lo ;
         O_par <= calc_prty(latched_data_lo) ;
         O_par64 <= calc_prty(latched_data_hi) ;
      end
end 


function calc_prty ;
input [31:0] rd_data ; // parity should be computed on the read data 
begin

   calc_prty = ^rd_data ;

end 
endfunction




endmodule 
